import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_counts_by_day_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/models/counts_by_day.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/counts_by_day.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardCountsByDay extends StatelessWidget {
  const CardCountsByDay({super.key});

  @override
  Widget build(BuildContext context) {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    final int daysCount = settings.getDistributionDaysCount();

    return BlocBuilder<DataCountsByDayCubit, DataCountsByDayState>(
      builder: (BuildContext context, DataCountsByDayState data) {
        final CountsByDayData counts = data.countsByDay ?? CountsByDayData.fromJson({});

        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'counts_by_day'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: update(daysCount),
          content: ChartCountsByDay(chartData: counts),
        );
      },
    );
  }

  Widget update(int daysCount) {
    const Widget loading = Text('⏳');
    const Widget done = Text('');

    late Future<CountsByDayData> future = ScrobblesApi.fetchCountsByDay(daysCount);

    return BlocBuilder<DataCountsByDayCubit, DataCountsByDayState>(
      builder: (BuildContext context, DataCountsByDayState data) {
        return FutureBuilder<CountsByDayData>(
          future: future,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

            BlocProvider.of<DataCountsByDayCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
