import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_top_artists_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/models/topartists.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/top_artists.dart';
import 'package:scrobbles/ui/widgets/charts/top_artists_stream.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardTopArtists extends StatelessWidget {
  const CardTopArtists({super.key});

  @override
  Widget build(BuildContext context) {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    final int daysCount = settings.getTopArtistsDaysCount();

    return BlocBuilder<DataTopArtistsCubit, DataTopArtistsState>(
      builder: (BuildContext context, DataTopArtistsState data) {
        final TopArtistsData artistsData = data.topArtists ?? TopArtistsData.fromJson({});

        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'top_artists_title'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: update(daysCount),
          content: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              ChartTopArtists(chartData: artistsData),
              const SizedBox(height: 8),
              ChartTopArtistsStream(chartData: artistsData),
            ],
          ),
        );
      },
    );
  }

  Widget update(int daysCount) {
    const Widget loading = Text('⏳');
    const Widget done = Text('');

    late Future<TopArtistsData> future = ScrobblesApi.fetchTopArtists(daysCount);

    return BlocBuilder<DataTopArtistsCubit, DataTopArtistsState>(
      builder: (BuildContext context, DataTopArtistsState update) {
        return FutureBuilder<TopArtistsData>(
          future: future,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

            BlocProvider.of<DataTopArtistsCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
